/*
 * Inspired from SIMILE's example at 
 * http://simile.mit.edu/wiki/Java_Firefox_Extension
 */
/*
 * The implementation consists of two parts: the module implementation, which 
 * acts like a factory for the components in the file, and the actual component.
 */ 
/*
 *  ATTENTION
 *
 *  Firefox is very very unforgiving to syntax errors in XPCOM Javascript
 *  files. If this file does not parse properly once, it might not be
 *  loaded again even if you restart Firefox. Watch the JavaScript
 *  Console for any error message (only logged the first time the file
 *  fails to parse).
 *
 *  You might have to uninstall the whole extension, restart Firefox, 
 *  reinstall the extension (with fixes), and then restart Firefox.
 */
 
function _printToJSConsole(msg) {
    Components.classes["@mozilla.org/consoleservice;1"]
        .getService(Components.interfaces.nsIConsoleService)
            .logStringMessage(msg);
}

/*----------------------------------------------------------------------
 * The Module (instance factory)
 *----------------------------------------------------------------------
 */

var BackendModule = {
    /*
     *  VERY IMPORTANT: Modify these first 3 fields to make them unique
     *  to your components. Note that these fields have nothing to
     *  do with the extension ID, nor the IDL interface IDs that the
     *  component implements. A component can implement several interfaces.
     */
    _myComponentID : Components.ID("{52c855f8-f64e-4f29-831a-cbc8ad79d347}"),
    _myName :        "Component connecting to java backend.",
    _myContractID :  "@jsug.tuwien.at/je-backend;1",
    
    /*
     *  This flag specifies whether this factory will create only a
     *  single instance of the component.
     */
    _singleton :     true,
    _myFactory : {
        createInstance : function(outer, iid) {
            if (outer != null) {
                throw Components.results.NS_ERROR_NO_AGGREGATION;
            }
            
            var instance = null;
            
            if (this._singleton) {
                instance = this.theInstance;
            }
            
            if (!(instance)) {
                instance = new BackendComponent(); 
            }
            
            if (this._singleton) {
                this.theInstance = instance;
            }

            return instance.QueryInterface(iid);
        }
    },
    
    registerSelf : function(compMgr, fileSpec, location, type) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.registerFactoryLocation(
            this._myComponentID,
            this._myName,
            this._myContractID,
            fileSpec,
            location,
            type
        );
    },

    unregisterSelf : function(compMgr, fileSpec, location) {
        compMgr = compMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
        compMgr.unregisterFactoryLocation(this._myComponentID, fileSpec);
    },

    getClassObject : function(compMgr, cid, iid) {
        if (cid.equals(this._myComponentID)) {
            return this._myFactory;
        } else if (!iid.equals(Components.interfaces.nsIFactory)) {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }

        throw Components.results.NS_ERROR_NO_INTERFACE;
    },

    canUnload : function(compMgr) {
        /*
         *  Do any unloading task you want here
         */
        return true;
    }
}

/*
 *  This function NSGetModule will be called by Firefox to retrieve the
 *  module object. This function has to have that name and it has to be
 *  specified for every single .JS file in the components directory of
 *  your extension.
 */
function NSGetModule(compMgr, fileSpec) {
    return BackendModule;
}


function BackendComponent() {
    /*
     *  This is a XPCOM-in-Javascript trick: Clients using an XPCOM
     *  implemented in Javascript can access its wrappedJSObject field
     *  and then from there, access its Javascript methods that are
     *  not declared in any of the IDL interfaces that it implements.
     *
     *  Being able to call directly the methods of a Javascript-based
     *  XPCOM allows clients to pass to it and receive from it
     *  objects of types not supported by IDL.
     */
    this.wrappedJSObject = this;
        
    this._initialized = false;
    // the java object -> this needs to be passed from the controller javascript
    this._java = null;
    // the backend
    this._backend = null;
}

/*
 *  nsISupports.QueryInterface
 */
BackendComponent.prototype.QueryInterface = function(iid) {
	/*
	  The component implements only the nsISupports interface.
	*/
    if (!iid.equals(Components.interfaces.nsISupports)) {
        throw Components.results.NS_ERROR_NO_INTERFACE;
    }
    return this;
};

/*
	This function initializez the BackendComponent, builds the ClassLoader
	for the java components, and stores the java backend as a member.
*/
BackendComponent.prototype.initialize = function (java, verbose) {
    if (this._initialized) {
        return true;
    }
    
    this._verboseFlag = (verbose);
    this._trace("BackendComponent.initialize");
	this._java = java;
    try {
		this.initializeJavaLibrary();
	    this._initialized = true;
    } catch(e) {
    	this._trace(e);
    }
    
    return this._initialized;
};

BackendComponent.prototype.initializeJavaLibrary = function() {
	// If verbose, get jvm console
	if (this._verboseFlag) {
	    var jvm = Components.classes["@mozilla.org/oji/jvm-mgr;1"]
	    					.getService(Components.interfaces.nsIJVMManager);
		jvm.showJavaConsole();
	}
	
	// Set no security
	this._java.lang.System.setSecurityManager(null);
					       
	// Get the extension directory
	var extensionID = "jsug@tuwien.at";
	var extensionDir = Components.classes["@mozilla.org/extensions/manager;1"]
					   	   .getService(Components.interfaces.nsIExtensionManager)
					       .getInstallLocation(extensionID)
					       .getItemLocation(extensionID);
  // Build path to the jars
	var libDir = extensionDir.clone();
	libDir.append("lib");
  var backendJarLocation = libDir.clone();
  
  // Dependency jars 
  	
  var jars = ["lingpipe-3.6.0.jar", "ganesha.jar"];
  
  var jarFiles = [];
  var tempJar;
    
  // Load jars
  for (var i = 0; i < jars.length; i++) {
    //tempJar = backendJarLocation.path + "\\" + jars[i];
    tempJar = backendJarLocation.clone();
    tempJar.append(jars[i]);
    this._trace("jar: " + tempJar.path + " loaded ");
    jarFiles[i] = new this._java.io.File(tempJar.path).toURL();
  }
  
  var classLoader = new this._java.net.URLClassLoader([jarFiles[0], jarFiles[1]]);
 
	var backendClass = this._java.lang.Class.forName(
			"backend.BackendHandler", 
			true, classLoader);
	
  // Make instance of the DictChunker class
  this._backend = backendClass.newInstance();
	
  // Set path of the dictionary xml
  var dictPath = backendJarLocation.clone();
  dictPath.append("dict.xml"); 
  
  // Set path of the websevice xml
  var wsPath = backendJarLocation.clone();
  wsPath.append("webservice.xml");
  
	this._backend.init(dictPath.path, wsPath.path);
	this._trace("Dictionary path set to: " + dictPath.path);
	this._trace("Webserwices path set to: " + wsPath.path);
};

/**
 * WEBSERVICES
 */ 
BackendComponent.prototype.getCategories = function() {
  return this._backend.getCategories();
}

BackendComponent.prototype.deleteCategory = function(name) {
  return this._backend.deleteCategory(name);
};

BackendComponent.prototype.getServices = function(categoryName) {
  return this._backend.getServices(categoryName);
}

BackendComponent.prototype.addService = function(category, name, url) {
  return this._backend.addService(category, name, url);
};

BackendComponent.prototype.deleteService = function(category, name) {
  return this._backend.deleteService(category, name);
};

/**
 * DICTIONARY
 */ 
BackendComponent.prototype.getEntities = function() {
  return this._backend.getEntities();
}

BackendComponent.prototype.getPhrases = function(entityName) {
  return this._backend.getPhrases(entityName);
}

BackendComponent.prototype.getRegexps = function(entityName) {
  return this._backend.getRegexps(entityName);
}

BackendComponent.prototype.deleteEntity = function(entityName) {
  return this._backend.deleteEntity(entityName);
}

BackendComponent.prototype.deletePhrases = function(entityName) {
  return this._backend.deletePhrases(entityName);
}

BackendComponent.prototype.deletePhrase = function(entityName, phrase) {
  return this._backend.deletePhrase(entityName, phrase);
}

BackendComponent.prototype.deleteRegexps = function(entityName) {
  return this._backend.deleteRegexps(entityName);
}

BackendComponent.prototype.deleteRegexp = function(entityName, regexp) {
  return this._backend.deleteRegexp(entityName, regexp);
}
 

/**
 * OTHERS
 */ 
BackendComponent.prototype.parse = function(pageSource) {
  return this._backend.parse(pageSource);
};

BackendComponent.prototype._trace = function(msg) {
  if (this._verboseFlag) {
      _printToJSConsole(msg);
  }
};

