package backend.dictionary;


import java.io.FileWriter;
import java.io.Writer;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Vector;

import javax.xml.bind.Marshaller;

import backend.BackendHandler;
import backend.dictionary.xml.EntityType;


public class DictManagement {
	
	/**
	 * This method marshalls the dictionary back to dict.xml
	 * @param bHandler Singleton object
	 */
	private static void saveDictionary(BackendHandler bHandler) {
		try {
			Marshaller m = bHandler.getDictContext().createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			Writer w = null;
			w = new FileWriter(bHandler.getDictfile());
			m.marshal(bHandler.getEntityMaster(), w);
			w.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	//=======
	// ENTITY
	//=======
	
	/**
	 * Adds an entity to the dictionary if it doesn't already exist.
	 * @param entityName Name of the entity
	 * @param bHandler Singleton object
	 */
	public static void addEntity(String entityName, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
		
		if (entity == null) {
			entity = new EntityType(entityName);
			bHandler.getEntityMaster().getEntityTypes().add(entity);
			saveDictionary(bHandler);
			System.out.println("Entity " + entityName + " added.");
			return;
		}
		
		System.out.println("Entity " + entityName + " already exists.");
	}
	
	/**
	 * Deletes entity from the dictionary, if exists.
	 * @param entityName Name of the entity
	 * @param bHandler Dingleton object
	 */
	public static void deleteEntity(String entityName, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
				
		if (entity != null) {
			bHandler.getEntityMaster().getEntityTypes().remove(entity);
			saveDictionary(bHandler);
			System.out.println("Entity " + entityName + " deleted.");
			return;
		}
		
		System.out.println("Entity " + entityName + " was not found.");
	}
	
	/**
	 * Returns all entityNames
	 * @param bHandler Singleton object
	 * @return Vector<String>
	 */
	public static Vector<String> getEntities(BackendHandler bHandler) {
		Vector<String> entities = new Vector<String>();
		for (Enumeration<EntityType> e = Collections.enumeration(bHandler.getEntityMaster().getEntityTypes()); e.hasMoreElements();) {
			entities.add(e.nextElement().getName());
		}
		
		return entities;
	}
	
	/**
	 * Search dictionary for specific entity.
	 * @param entityName Name of the entity
	 * @param bHandler Singleton object
	 * @return EntityType
	 */
	public static EntityType getEntity(String entityName, BackendHandler bHandler) {
		EntityType entity = null;
		
		for (Enumeration<EntityType> e = Collections.enumeration(bHandler.getEntityMaster().getEntityTypes()); e.hasMoreElements();) {
			entity = e.nextElement();
			if (entity.getName().equals(entityName)) {
				return entity;
			}
		}
		
		return null;
	}
		
	
	//==================
	// PHRASE FOR ENTITY
	//==================
	
	/**
	 * Adds a new Entity-Type to the database or adds a new phrase to the given Entity Type.
	 * @param entityType Type of the entity
	 * @param entityPhrase Phrase to be added
	 * @param bHandler Singleton object
	 */
	public static void addPhrase(String entityName, String phrase, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
		
		//-- If entity does not exist
		if (entity == null) {
			entity = new EntityType(entityName);
			entity.getPhrase().add(phrase);
			bHandler.getEntityMaster().getEntityTypes().add(entity);
			saveDictionary(bHandler);
			System.out.println("Entity " + entityName + " created and phrase " + phrase + " added.");
		//-- If entity exists
		} else {
			//-- If phrase already contained in entity
			for (Enumeration<String> e = Collections.enumeration(entity.getPhrase()); e.hasMoreElements();) {
				if (e.nextElement().equals(phrase)) {
					System.out.println("Entity " + entityName + " already contains phrase " + phrase + ".");
					return;
				}
			}
			
			entity.getPhrase().add(phrase);
			saveDictionary(bHandler);
			System.out.println("Phrase " + phrase + " added to entity " + entityName + ".");
		}
	}
	
	/**
	 * Deletes a phrase for an entity if exists.
	 * @param entityName Name of the entity
	 * @param phrase Phrase to be deleted
	 * @param bHandler Singleton object
	 */
	public static void deletePhrase(String entityName, String phrase, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
				
		if (entity != null) {
			for (Enumeration<String> e = Collections.enumeration(entity.getPhrase()); e.hasMoreElements();) {
				if (e.nextElement().equals(phrase)) {
					entity.getPhrase().remove(phrase);
					saveDictionary(bHandler);
					System.out.println("Phrase " + phrase + " for entity " + entityName + " deleted.");
					break;
				}
			}
		}
		
		System.out.println("Entity " + entityName + " with phrase " + phrase + " was not found.");
	}
	
	/**
	 * Deletes all phrases for specified entity.
	 * @param entityName Name of the entity
	 * @param bHandler Singleton object
	 */
	public static void deletePhrases(String entityName, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
		if (entity.getPhrase().size() == 0) {
			System.out.println("Entity " + entityName + " does not have any phrases.");
			return;
		}
		entity.setPhrase(new Vector<String>());
		saveDictionary(bHandler);
		System.out.println("All phrases for entity " + entityName + " deleted.");
	}
	
	/**
	 * Returns phrases for an entity.
	 * @param entity which phrases are to be returned
	 * @param bHandler Singleton object
	 * @return Vector<String>
	 */
	public static Vector<String> getPhrases(String entityName, BackendHandler bHandler) {
		Vector<String> phrases = new Vector<String>();
		EntityType entity = getEntity(entityName, bHandler);
		if (entity != null) {
			for (Enumeration<String> e2 = Collections.enumeration(entity.getPhrase()); e2.hasMoreElements();) {
				phrases.add(e2.nextElement());
			}
		}
		return phrases;
	}
	
	//==================
	// REGEXP FOR ENTITY
	//==================

	/**
	 * Adds a new Entity Type with a regex to the Database, if the entity Type already exists and
	 * has a regex, the regex will be overwritten
	 * @param entityType Type of the Entity
	 * @param regex Regular expression
	 * @param bHandler Singleton object
	 */
	public static void addRegexType(String entityName, String regex, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
		
		//-- If entity does not exist
		if (entity == null) {
			entity = new EntityType(entityName);
			entity.getRegex().add(regex);
			bHandler.getEntityMaster().getEntityTypes().add(entity);
			saveDictionary(bHandler);
			System.out.println("Entity " + entityName + " created and regex " + regex + " added.");
		//-- If entity exists
		} else {
			//-- If regex already contained in entity
			for (Enumeration<String> e = Collections.enumeration(entity.getRegex()); e.hasMoreElements();) {
				if (e.nextElement().equals(regex)) {
					System.out.println("Entity " + entityName + " already contains regex " + regex + ".");
					return;
				}
			}
			
			entity.getRegex().add(regex);
			saveDictionary(bHandler);
			System.out.println("Regex " + regex + " added to entity " + entityName + ".");
		}
		
	}
	
	/**
	 * Deletes regexp for an entity if exists.
	 * @param entityName Name of the entity
	 * @param regexp regexp to be deleted
	 * @param bHandler Singleton object
	 */
	public static void deleteRegexp(String entityName, String regexp, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
				
		if (entity != null) {
			for (Enumeration<String> e = Collections.enumeration(entity.getRegex()); e.hasMoreElements();) {
				if (e.nextElement().equals(regexp)) {
					entity.getRegex().remove(regexp);
					saveDictionary(bHandler);
					System.out.println("Regex " + regexp + " for entity " + entityName + " deleted.");
					break;
				}
			}
		}
		
		System.out.println("Entity " + entityName + " with regex " + regexp + " was not found.");
	}
	
	/**
	 * Deletes all regexps for specified entity.
	 * @param entityName Name of the entity
	 * @param bHandler Singleton object
	 */
	public static void deleteRegexps(String entityName, BackendHandler bHandler) {
		EntityType entity = getEntity(entityName, bHandler);
		if (entity.getRegex().size() == 0) {
			System.out.println("Entity " + entityName + " does not have any regexps.");
			return;
		}
		entity.setRegex(new Vector<String>());
		saveDictionary(bHandler);
		System.out.println("All regexps for entity " + entityName + " deleted.");
	}
	
	/**
	 * Returns regexs for an entity.
	 * @param entityName Name of the entity
	 * @param bHandler Singleton object
	 * @return Vector<String>
	 */
	public static Vector<String> getRegexps(String entityName, BackendHandler bHandler) {
		Vector<String> regexps = new Vector<String>();
		EntityType entity = getEntity(entityName, bHandler);
		
		for (Enumeration<String> e2 = Collections.enumeration(entity.getRegex()); e2.hasMoreElements();) {
			regexps.add(e2.nextElement());
		}
		
		return regexps;
	}
}
