package backend.webservice;

import java.io.FileWriter;
import java.io.Writer;
import java.util.Collections;
import java.util.Enumeration;
import java.util.Vector;

import javax.xml.bind.Marshaller;

import backend.BackendHandler;
import backend.webservice.xml.WSCategory;
import backend.webservice.xml.WebService;

public class WSManagement {

	/**
	 * This method marshalls the webservices back to webservice.xml
	 * @param bHandler Singleton object
	 */
	private static void saveServices(BackendHandler bHandler) {
		try {
			Marshaller m = bHandler.getWsContext().createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
			Writer w = null;
			w = new FileWriter(bHandler.getWsfile());
			m.marshal(bHandler.getWsRootElement(), w);
			w.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	//=========
	// CATEGORY
	//=========
	
	/**
	 * Adds a new service to the webservice xml file
	 * @param name Name of the service
	 * @param bHandler Singleton object
	 */
	public static void addCategory(String name, BackendHandler bHandler) {
		WSCategory category = getCategory(name, bHandler);
		if (category != null) {
			System.out.println("Category " + name + " already exists.");
		} else {
			category = new WSCategory(name);
			bHandler.getWsRootElement().getCategories().add(category);
			saveServices(bHandler);
			System.out.println("Category " + name + " added.");
		}
	}
	
	/**
	 * Deletes category from the webservice xml file
	 * @param name Name of the category
	 * @param bHandler Singleton object
	 */
	public static void deleteCategory(String name, BackendHandler bHandler) {
		WSCategory category = getCategory(name, bHandler);
		if (category != null) {
			bHandler.getWsRootElement().getCategories().remove(category);
			saveServices(bHandler);
			System.out.println("Category " + name + " deleted.");
		} else {
			System.out.println("Category " +  name + " not found.");
		}
	}
	
	/**
	 * Reads all categories from webservice xml file
	 * @param bHandler Singleton object
	 * @return Vector<String>
	 */
	public static Vector<String> getCategories(BackendHandler bHandler) {
		Vector<String> categories = new Vector<String>();
		for (Enumeration<WSCategory> e = Collections.enumeration(bHandler.getWsRootElement().getCategories()); e.hasMoreElements();) {
			categories.add(e.nextElement().getName());
		}
		
		return categories;
	}
	
	/**
	 * Searches for specified category
	 * @param name Name of the category
	 * @param bHandler Singleton object
	 * @return WSCategory
	 */
	private static WSCategory getCategory(String name, BackendHandler bHandler) {
		WSCategory category = null;
		
		for (Enumeration<WSCategory> e = Collections.enumeration(bHandler.getWsRootElement().getCategories()); e.hasMoreElements();) {
			category = e.nextElement();
			if (category.getName().equals(name)) {
				return category; 
			}
		}
		
		return null;
	}
	
	//=====================
	// SERVICE FOR CATEGORY
	//=====================

	/**
	 * Adds new webservice to a category
	 * @param category Name of the category
	 * @param name Name of the webservice
	 * @param url URL of the webservice
	 * @param bHandler Singleton object
	 */
	public static void addService(String category, String name, String url, BackendHandler bHandler) {
		WSCategory cat = getCategory(category, bHandler);
		
		//-- If category does not exist
		if (cat == null) {
			cat = new WSCategory(category);
			cat.getServices().add(new WebService(name, url));
			bHandler.getWsRootElement().getCategories().add(cat);
			saveServices(bHandler);
			System.out.println("Category " + category + " created and service " + name + " added.");
		//-- If category exists
		} else {
			for (Enumeration<WebService> e = Collections.enumeration(cat.getServices()); e.hasMoreElements();) {
				//-- If service already contained in category
				if (e.nextElement().getName().equals(name)) {
					System.out.println("Service " + name + " already exists in category " + category + ".");
					return;
				}
			}
			
			cat.getServices().add(new WebService(name, url));
			//bHandler.getWsRootElement().getCategories().add(cat);
			saveServices(bHandler);
			System.out.println("Service " + name + " added to category " + category + ".");
		}
	}
	
	/**
	 * Deletes service from a category
	 * @param category Name of the category
	 * @param name Name of the webservice
	 * @param bHandler Singleton object
	 */
	public static void deleteService(String category, String name, BackendHandler bHandler) {
		WSCategory cat = getCategory(category, bHandler);
		
		//-- Category not found
		if (cat == null) {
			System.out.println("Category " + category + " does not exist.");
		//-- Category found
		} else {
			for (Enumeration<WebService> e = Collections.enumeration(cat.getServices()); e.hasMoreElements();) {
				WebService ws = e.nextElement();
				//-- Service found
				if (ws.getName().equals(name)) {
					cat.getServices().remove(ws);
					saveServices(bHandler);
					System.out.println("Service " + name + " deleted from category " + category + ".");
					return;
				}
			}
			//-- Service not found
			System.out.println("Service " + name + " in category " + category + " not found.");
		}
		
		
	}
	
	/**
	 * Returns all services for a category
	 * @param category Name of the category
	 * @param bHandler Singleton object
	 * @return Vector<WebService>
	 */
	public static Vector<WebService> getServices(String category, BackendHandler bHandler) {
		Vector<WebService> services = new Vector<WebService>();
		WSCategory cat = getCategory(category, bHandler);
		if (cat != null) {
			for (Enumeration<WebService> e = Collections.enumeration(cat.getServices()); e.hasMoreElements();) {
				services.add(e.nextElement());
			}
		}
		return services;
	}
}
